{*************************************************************}
{            TTaskBar Component for Delphi IV                 }
{ Version:   1.01                                             }
{ Author:    Aleksey Kuznetsov                                }
{ E-Mail:    info@utilmind.com                                }
{ Homepage:  http://www.utilmind.com                          }
{ Created:   January 25, 1999                                 }
{ Modified:  February 12, 1999                                }
{ Legal:     Copyright (c) 1999, UtilMind Solutions           }
{-------------------------------------------------------------}
{ Thanks:    Markus Stephany (mirbir.st@t-online.de)          }
{                http://home.t-online.de/home/mirbir.st       }
{*************************************************************}
{ TaskBar Panel - inherited from TToolBar. Added some useful  }
{ functions. See example (MDIApp).                            }
{*************************************************************}
{                    IMPORTANT NOTE:                          }
{  This code may be used and modified by anyone so long as    }
{ this header and copyright information remains intact. By    }
{ using this code you agree to indemnify UtilMind Solutions   }
{ from any liability that might arise from its use. You must  }
{ obtain written consent before selling or redistributing     }
{ this code.                                                  }
{*************************************************************}
{  Properties:                                                }
{          Count: Indicates the number of entries (tasks) in  }
{                 the TaskList.                               }
{         Showed: VISIBLE (!!!) TaskBar can be not always     }
{                 visible.                                    }
{     Methods:                                                }
{            Add: Adds Task to TaskList.                      }
{         Remove: Removes Task from TaskList.                 }
{ RedrawCaptions: Redraws all captions (if they changed).     }
{*************************************************************}
{ FOR MUCH MORE INFORMATION - SEE DEMO PROGRAM.               }
{*************************************************************}
{ Thanks for using TTaskBar component.                        }
{ If at occurrence of any questions concerning these          }
{ components, drop us mail: info@utilmind.com                 }
{*************************************************************}
unit TaskBar;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ToolWin, ComCtrls;

type
  TTask = class(TObject)
     Task: TForm;
     ToolButton: TToolButton;
     ParentBar: TToolBar;

     TaskList: TList;  { Parent }
     TaskOnActivate: TNotifyEvent;

     constructor Create(aTask: TForm; ImageIndex: Integer; BtnParent: TToolBar; aTaskList: TList);
     destructor Destroy; override;

     procedure ClickButton(Sender: TObject);
     procedure ButtonUp;
   end;

  TOnActivateTask = procedure(Sender: TObject; Task: TForm) of object;

  TTaskBar = class(TToolBar)
  private
    FCount: Integer;
    FShowed: Boolean;
    TaskList: TList;

    procedure SetShowed(Value: Boolean);
  protected
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;

    procedure Add(Task: TForm; ImageIndex: Integer);
    procedure Remove(Task: TForm);
    procedure RedrawCaptions;
  published
    property Count: Integer read FCount;
    property Showed: Boolean read FShowed write SetShowed;
  end;

procedure Register;

implementation

constructor TTask.Create(aTask: TForm; ImageIndex: Integer; BtnParent: TToolBar; aTaskList: TList);
begin
  Task := aTask;
  TaskOnActivate := Task.OnActivate;
  Task.OnActivate := ClickButton;
  TaskList := aTaskList;
  ButtonUp;
  ToolButton := TToolButton.Create(BtnParent);
  ToolButton.Caption := aTask.Caption;
  ToolButton.ImageIndex := ImageIndex;
  ToolButton.OnClick := ClickButton;
  BtnParent.InsertControl(ToolButton);
  ToolButton.Grouped := True;
  ToolButton.AllowAllUp := True;
  ToolButton.AutoSize := True;
  ToolButton.Down := True;
  ParentBar := BtnParent;
end;

destructor TTask.Destroy;
begin
  ParentBar.RemoveControl(ToolButton);
  ToolButton.Free;
  inherited Destroy;
end;

procedure TTask.ClickButton(Sender: TObject);
var
  i: Integer;
  CurrentTask: TTask;
begin
  ButtonUp;
  if (Sender.ClassName = 'TToolButton') then
   (Sender as TToolButton).Down := True
  else
   begin
    i := TaskList.Count;
    if i <> 0 then
     for i := 0 to i - 1 do
      begin
       CurrentTask := TaskList[i];
       if CurrentTask.Task = Sender then
        begin
         CurrentTask.ToolButton.Down := True;
         Break;
        end;
      end;
   end;
  SendMessage(Task.Handle, wm_NCActivate, wa_Active, 0);

{ Thanks Markus Stephany for next line:
 MS> when i changed to a different mdi child with e.g.
 MS> a memo control on it, the caret was not visible every time }
  SendMessage(Task.Handle, WM_ChildActivate, 0, 0);
   
  Task.SetFocus;
  Task.BringToFront;

  // Restore from icon
  if Task.WindowState = wsMinimized then
   Task.WindowState := wsNormal;

  if Assigned(TaskOnActivate) then
   TaskOnActivate(Sender);
end;

procedure TTask.ButtonUp;
var
  i: Integer;
  CurrentTask: TTask;
begin
  i := TaskList.Count;
  if i <> 0 then
   for i := 0 to i - 1 do
    begin
     CurrentTask := TaskList[i];
     if CurrentTask.ToolButton.Down then
      CurrentTask.ToolButton.Down := False;
    end;
end;

{ TTaskBar }

constructor TTaskBar.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  ShowCaptions := True;
  List := True;
  Visible := False;
  FShowed := True;
  TaskList := TList.Create;
end;

destructor TTaskBar.Destroy;
begin
  TaskList.Free;
  inherited Destroy;
end;

procedure TTaskBar.Add(Task: TForm; ImageIndex: Integer);
var
  i: Integer;
  CurrentTask: TTask;
begin
  if Task <> nil then
   begin
    i := TaskList.Count;
    if i <> 0 then
     for i := 0 to i - 1 do
      begin
       CurrentTask := TaskList[i];
       if CurrentTask.Task = Task then Exit;
      end;
    CurrentTask := TTask.Create(Task, ImageIndex, Self, TaskList);
    TaskList.Add(CurrentTask);
    FCount := TaskList.Count;
    if (FCount > 0) and not Visible and FShowed then Visible := True;
   end;
end;

procedure TTaskBar.Remove(Task: TForm);
var
  i: Integer;
  CurrentTask: TTask;
begin
  if Task <> nil then
   begin
    i := TaskList.Count;
    if i <> 0 then
     for i := 0 to i - 1 do
      begin
       CurrentTask := TaskList[i];
       if CurrentTask.Task = Task then
        begin
         TaskList.Remove(CurrentTask);
         CurrentTask.Free;
         Break;
        end;
      end;
    FCount := TaskList.Count;
    if (FCount = 0) and Visible then Visible := False;
   end;
end;

procedure TTaskBar.RedrawCaptions;
var
  i: Integer;
  CurrentTask: TTask;
begin
  if FCount <> 0 then
   for i := 0 to FCount - 1 do
    begin
     CurrentTask := TaskList[i];
     with CurrentTask do
      if Task.Caption <> ToolButton.Caption then
       ToolButton.Caption := Task.Caption;
    end;
end;

procedure TTaskBar.SetShowed(Value: Boolean);
begin
  if FShowed <> Value then
   begin
    if not FShowed and (FCount > 0) then
     Visible := True
    else Visible := False;
    
    FShowed := Value;
   end;
end;

procedure Register;
begin
  RegisterComponents('Xacker', [TTaskBar]);
end;

end.
